@@ -1,4 +1,4 @@
-# $Id: ODBC.pm 12674 2009-04-07 09:04:09Z mjevans $
+# $Id: ODBC.pm 13205 2009-08-19 08:51:11Z mjevans $
 #
 # Copyright (c) 1994,1995,1996,1998  Tim Bunce
 # portions Copyright (c) 1997-2004  Jeff Urlwin
@@ -8,13 +8,15 @@
 # You may distribute under the terms of either the GNU General Public
 # License or the Artistic License, as specified in the Perl README file.
 
-## no critic (ProhibitManyArgs)
+## no critic (ProhibitManyArgs ProhibitMultiplePackages)
 
 require 5.006;
 
-$DBD::ODBC::VERSION = '1.21';
+$DBD::ODBC::VERSION = '1.23';
 
 {
+    ## no critic (ProhibitMagicNumbers ProhibitExplicitISA)
+    ## no critic (ProhibitPackageVars)
     package DBD::ODBC;
 
     use DBI ();
@@ -23,21 +25,21 @@ $DBD::ODBC::VERSION = '1.21';
 
     @ISA = qw(Exporter DynaLoader);
 
-    # my $Revision = substr(q$Id: ODBC.pm 12674 2009-04-07 09:04:09Z mjevans $, 13,2);
+    # my $Revision = substr(q$Id: ODBC.pm 13205 2009-08-19 08:51:11Z mjevans $, 13,2);
 
     require_version DBI 1.21;
 
     bootstrap DBD::ODBC $VERSION;
 
     $err = 0;           # holds error code   for DBI::err
-    $errstr = "";       # holds error string for DBI::errstr
+    $errstr = q{};              # holds error string for DBI::errstr
     $sqlstate = "00000";
     $drh = undef;       # holds driver handle once initialised
 
     sub parse_trace_flag {
         my ($class, $name) = @_;
-        return 0x02000000 if $name eq 'odbcunicode';
-        return 0x04000000 if $name eq 'odbcconnection';
+        return 0x02_00_00_00 if $name eq 'odbcunicode';
+        return 0x04_00_00_00 if $name eq 'odbcconnection';
         return DBI::parse_trace_flag($class, $name);
     }
 
@@ -60,10 +62,10 @@ $DBD::ODBC::VERSION = '1.21';
 	    'Err'    => \$DBD::ODBC::err,
 	    'Errstr' => \$DBD::ODBC::errstr,
 	    'State' => \$DBD::ODBC::sqlstate,
-	    'Attribution' => 'ODBC DBD by Jeff Urlwin',
+	    'Attribution' => 'DBD::ODBC by Jeff Urlwin, Tim Bunce and Martin J. Evans',
 	    });
 
-	$drh;
+	return $drh;
     }
 
     sub CLONE { undef $drh }
@@ -77,10 +79,9 @@ $DBD::ODBC::VERSION = '1.21';
 
     ## no critic (ProhibitBuiltinHomonyms)
     sub connect {
-	my $drh = shift;
-	my($dbname, $user, $auth, $attr)= @_;
-	$user = '' unless defined $user;
-	$auth = '' unless defined $auth;
+	my($drh, $dbname, $user, $auth, $attr)= @_;
+	#$user = q{} unless defined $user;
+	#$auth = q{} unless defined $auth;
 
 	# create a 'blank' dbh
 	my $this = DBI::_new_dbh($drh, {
@@ -89,12 +90,16 @@ $DBD::ODBC::VERSION = '1.21';
 	    'CURRENT_USER' => $user,
 	    });
 
-	# Call ODBC logon func in ODBC.xs file
+	# Call ODBC _login func in Driver.xst file => dbd_db_login6
 	# and populate internal handle data.
+	# There are 3 versions (currently) if you have a recent DBI:
+	# dbd_db_login (oldest)
+	# dbd_db_login6 (with attribs hash & char * args) and
+	# dbd_db_login6_sv (as dbd_db_login6 with perl scalar args
 
 	DBD::ODBC::db::_login($this, $dbname, $user, $auth, $attr) or return;
 
-	$this;
+	return $this;
     }
     ## use critic
 
@@ -105,6 +110,11 @@ $DBD::ODBC::VERSION = '1.21';
     use strict;
     use warnings;
 
+    use constant SQL_DRIVER_HSTMT => 5;
+    use constant SQL_DRIVER_HLIB => 76;
+    use constant SQL_DRIVER_HDESC => 135;
+
+
     sub parse_trace_flag {
         my ($h, $name) = @_;
         return DBD::ODBC->parse_trace_flag($name);
@@ -133,7 +143,7 @@ $DBD::ODBC::VERSION = '1.21';
     sub prepare {
 	my($dbh, $statement, @attribs)= @_;
 
-	# create a 'blank' dbh
+	# create a 'blank' sth
 	my $sth = DBI::_new_sth($dbh, {
 	    'Statement' => $statement,
 	    });
@@ -145,44 +155,44 @@ $DBD::ODBC::VERSION = '1.21';
 	DBD::ODBC::st::_prepare($sth, $statement, @attribs)
 	    or return;
 
-	$sth;
+	return $sth;
     }
 
     sub column_info {
 	my ($dbh, $catalog, $schema, $table, $column) = @_;
 
-	$catalog = "" if (!$catalog);
-	$schema = "" if (!$schema);
-	$table = "" if (!$table);
-	$column = "" if (!$column);
+	$catalog = q{} if (!$catalog);
+	$schema = q{} if (!$schema);
+	$table = q{} if (!$table);
+	$column = q{} if (!$column);
 	# create a "blank" statement handle
 	my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLColumns" });
 
 	_columns($dbh,$sth, $catalog, $schema, $table, $column)
 	    or return;
 
-	$sth;
+	return $sth;
     }
 
     sub columns {
 	my ($dbh, $catalog, $schema, $table, $column) = @_;
 
-	$catalog = "" if (!$catalog);
-	$schema = "" if (!$schema);
-	$table = "" if (!$table);
-	$column = "" if (!$column);
+	$catalog = q{} if (!$catalog);
+	$schema = q{} if (!$schema);
+	$table = q{} if (!$table);
+	$column = q{} if (!$column);
 	# create a "blank" statement handle
 	my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLColumns" });
 
 	_columns($dbh,$sth, $catalog, $schema, $table, $column)
 	    or return;
 
-	$sth;
+	return $sth;
     }
 
 
     sub table_info {
- 	my($dbh, $catalog, $schema, $table, $type) = @_;
+ 	my ($dbh, $catalog, $schema, $table, $type) = @_;
 
 	if ($#_ == 1) {
 	   my $attrs = $_[1];
@@ -192,17 +202,17 @@ $DBD::ODBC::VERSION = '1.21';
 	   $type = $attrs->{TABLE_TYPE};
  	}
 
-	$catalog = "" if (!$catalog);
-	$schema = "" if (!$schema);
-	$table = "" if (!$table);
-	$type = "" if (!$type);
+	$catalog = q{} if (!$catalog);
+	$schema = q{} if (!$schema);
+	$table = q{} if (!$table);
+	$type = q{} if (!$type);
 
 	# create a "blank" statement handle
 	my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLTables" });
 
 	DBD::ODBC::st::_tables($dbh,$sth, $catalog, $schema, $table, $type)
 	      or return;
-	$sth;
+	return $sth;
     }
 
     sub primary_key_info {
@@ -211,12 +221,12 @@ $DBD::ODBC::VERSION = '1.21';
        # create a "blank" statement handle
        my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLPrimaryKeys" });
 
-       $catalog = "" if (!$catalog);
-       $schema = "" if (!$schema);
-       $table = "" if (!$table);
+       $catalog = q{} if (!$catalog);
+       $schema = q{} if (!$schema);
+       $table = q{} if (!$table);
        DBD::ODBC::st::_primary_keys($dbh,$sth, $catalog, $schema, $table )
 	     or return;
-       $sth;
+       return $sth;
     }
     sub statistics_info {
        my ($dbh, $catalog, $schema, $table, $unique, $quick ) = @_;
@@ -224,16 +234,16 @@ $DBD::ODBC::VERSION = '1.21';
        # create a "blank" statement handle
        my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLStatistics" });
 
-       $catalog = "" if (!$catalog);
-       $schema = "" if (!$schema);
-       $table = "" if (!$table);
+       $catalog = q{} if (!$catalog);
+       $schema = q{} if (!$schema);
+       $table = q{} if (!$table);
        $unique = 1 if (!$unique);
        $quick = 1 if (!$quick);
 
        DBD::ODBC::st::_statistics($dbh, $sth, $catalog, $schema, $table,
                                  $unique, $quick)
 	     or return;
-       $sth;
+       return $sth;
     }
 
     sub foreign_key_info {
@@ -242,84 +252,100 @@ $DBD::ODBC::VERSION = '1.21';
        # create a "blank" statement handle
        my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLForeignKeys" });
 
-       $pkcatalog = "" if (!$pkcatalog);
-       $pkschema = "" if (!$pkschema);
-       $pktable = "" if (!$pktable);
-       $fkcatalog = "" if (!$fkcatalog);
-       $fkschema = "" if (!$fkschema);
-       $fktable = "" if (!$fktable);
+       $pkcatalog = q{} if (!$pkcatalog);
+       $pkschema = q{} if (!$pkschema);
+       $pktable = q{} if (!$pktable);
+       $fkcatalog = q{} if (!$fkcatalog);
+       $fkschema = q{} if (!$fkschema);
+       $fktable = q{} if (!$fktable);
        _GetForeignKeys($dbh, $sth, $pkcatalog, $pkschema, $pktable, $fkcatalog, $fkschema, $fktable) or return;
-       $sth;
+       return $sth;
     }
 
     sub ping {
 	my $dbh = shift;
-	my $state = undef;
 
- 	my ($catalog, $schema, $table, $type);
+        # DBD::Gofer does the following (with a 0 instead of "0") but it I
+        # cannot make it set a warning.
+        #return $dbh->SUPER::set_err("0", "can't ping while not connected") # warning
+        #    unless $dbh->SUPER::FETCH('Active');
 
-	$catalog = "";
-	$schema = "";
-	$table = "NOXXTABLE";
-	$type = "";
+        #my $pe = $dbh->FETCH('PrintError');
+        #$dbh->STORE('PrintError', 0);
+        my $evalret = eval {
+           # create a "blank" statement handle
+            my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLTables_PING" })
+                or return 1;
 
-	# create a "blank" statement handle
-	my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLTables_PING" });
+            my ($catalog, $schema, $table, $type);
 
-	DBD::ODBC::st::_tables($dbh,$sth, $catalog, $schema, $table, $type)
-	      or return 0;
-	$sth->finish;
-	return 1;
+            $catalog = q{};
+            $schema = q{};
+            $table = 'NOXXTABLE';
+            $type = q{};
 
+            DBD::ODBC::st::_tables($dbh,$sth, $catalog, $schema, $table, $type)
+                  or return 1;
+            $sth->finish;
+            return 0;
+        };
+        #$dbh->STORE('PrintError', $pe);
+        $dbh->set_err(undef,'',''); # clear any stored error from eval above
+        if ($evalret == 0) {
+            return 1;
+        } else {
+            return 0;
+        }
     }
 
-    # saved, just for posterity.
-    sub oldping  {
-	my $dbh = shift;
-	my $state = undef;
+#####    # saved, just for posterity.
+#####    sub oldping  {
+#####	my $dbh = shift;
+#####	my $state = undef;
+#####
+#####	# should never 'work' but if it does, that's okay!
+#####	# JLU incorporated patches from Jon Smirl 5/4/99
+#####	{
+#####	    local $dbh->{RaiseError} = 0 if $dbh->{RaiseError};
+#####	    # JLU added local PrintError handling for completeness.
+#####	    # it shouldn't print, I think.
+#####	    local $dbh->{PrintError} = 0 if $dbh->{PrintError};
+#####	    my $sql = "select sysdate from dual1__NOT_FOUND__CANNOT";
+#####	    my $sth = $dbh->prepare($sql);
+#####	    # fixed "my" $state = below.  Was causing problem with
+#####	    # ping!  Also, fetching fields as some drivers (Oracle 8)
+#####	    # may not actually check the database for activity until
+#####	    # the query is "described".
+#####	    # Right now, Oracle8 is the only known version which
+#####	    # does not actually check the server during prepare.
+#####	    my $ok = $sth && $sth->execute();
+#####
+#####	    $state = $dbh->state;
+#####	    $DBD::ODBC::err = 0;
+#####	    $DBD::ODBC::errstr = "";
+#####	    $DBD::ODBC::sqlstate = "00000";
+#####	    return 1 if $ok;
+#####	}
+#####        return 1 if $state eq 'S0002';  # Base table not found
+##### 	return 1 if $state eq '42S02';  # Base table not found.Solid EE v3.51
+#####        return 1 if $state eq 'S0022';  # Column not found
+#####	return 1 if $state eq '37000';  # statement could not be prepared (19991011, JLU)
+#####	# return 1 if $state eq 'S1000';  # General Error? ? 5/30/02, JLU.  This is what Openlink is returning
+#####	# We assume that any other error means the database
+#####	# is no longer connected.
+#####	# Some special cases may need to be added to the code above.
+#####	return 0;
+#####    }
 
-	# should never 'work' but if it does, that's okay!
-	# JLU incorporated patches from Jon Smirl 5/4/99
-	{
-	    local $dbh->{RaiseError} = 0 if $dbh->{RaiseError};
-	    # JLU added local PrintError handling for completeness.
-	    # it shouldn't print, I think.
-	    local $dbh->{PrintError} = 0 if $dbh->{PrintError};
-	    my $sql = "select sysdate from dual1__NOT_FOUND__CANNOT";
-	    my $sth = $dbh->prepare($sql);
-	    # fixed "my" $state = below.  Was causing problem with
-	    # ping!  Also, fetching fields as some drivers (Oracle 8)
-	    # may not actually check the database for activity until
-	    # the query is "described".
-	    # Right now, Oracle8 is the only known version which
-	    # does not actually check the server during prepare.
-	    my $ok = $sth && $sth->execute();
-
-	    $state = $dbh->state;
-	    $DBD::ODBC::err = 0;
-	    $DBD::ODBC::errstr = "";
-	    $DBD::ODBC::sqlstate = "00000";
-	    return 1 if $ok;
-	}
-        return 1 if $state eq 'S0002';  # Base table not found
- 	return 1 if $state eq '42S02';  # Base table not found.Solid EE v3.51
-        return 1 if $state eq 'S0022';  # Column not found
-	return 1 if $state eq '37000';  # statement could not be prepared (19991011, JLU)
-	# return 1 if $state eq 'S1000';  # General Error? ? 5/30/02, JLU.  This is what Openlink is returning
-	# We assume that any other error means the database
-	# is no longer connected.
-	# Some special cases may need to be added to the code above.
-	return 0;
-    }
-
-    # New support for the next DBI which will have a get_info command.
-    # leaving support for ->func(xxx, GetInfo) (above) for a period of time
+    # New support for DBI which has the get_info command.
+    # leaving support for ->func(xxx, GetInfo) (below) for a period of time
     # to support older applications which used this.
     sub get_info {
 	my ($dbh, $item) = @_;
-	# handle SQL_DRIVER_HSTMT, SQL_DRIVER_HLIB and
-	# SQL_DRIVER_HDESC specially
-	if ($item == 5 || $item == 135 || $item == 76) {
+	# Ignore some we cannot do
+	if ($item == SQL_DRIVER_HSTMT ||
+                $item == SQL_DRIVER_HLIB ||
+                    $item == SQL_DRIVER_HDESC) {
 	   return;
 	}
 	return _GetInfo($dbh, $item);
@@ -332,17 +358,18 @@ $DBD::ODBC::VERSION = '1.21';
     sub do {
         my($dbh, $statement, $attr, @params) = @_;
         my $rows = 0;
+        ## no critic (ProhibitMagicNumbers)
         if( -1 == $#params )
         {
           # No parameters, use execute immediate
           $rows = ExecDirect( $dbh, $statement );
           if( 0 == $rows )
           {
-            $rows = "0E0";
+              $rows = "0E0";    # 0 but true
           }
           elsif( $rows < -1 )
           {
-            undef $rows;
+              undef $rows;
           }
         }
         else
@@ -359,7 +386,7 @@ $DBD::ODBC::VERSION = '1.21';
     #
     sub ExecDirect {
        my ($dbh, $sql) = @_;
-       _ExecDirect($dbh, $sql);
+       return _ExecDirect($dbh, $sql);
     }
 
     # Call the ODBC function SQLGetInfo
@@ -370,7 +397,7 @@ $DBD::ODBC::VERSION = '1.21';
     #
     sub GetInfo {
 	my ($dbh, $item) = @_;
-	get_info($dbh, $item);
+	return get_info($dbh, $item);
     }
 
     # Call the ODBC function SQLStatistics
@@ -378,11 +405,12 @@ $DBD::ODBC::VERSION = '1.21';
     # See the ODBC documentation for more information about this call.
     #
     sub GetStatistics {
-        my ($dbh, $Catalog, $Schema, $Table, $Unique) = @_;
+        my ($dbh, $catalog, $schema, $table, $unique) = @_;
         # create a "blank" statement handle
         my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLStatistics" });
-        _GetStatistics($dbh, $sth, $Catalog, $Schema, $Table, $Unique) or return;
-        $sth;
+        _GetStatistics($dbh, $sth, $catalog, $schema,
+		       $table, $unique) or return;
+        return $sth;
     }
 
     # Call the ODBC function SQLForeignKeys
@@ -390,11 +418,13 @@ $DBD::ODBC::VERSION = '1.21';
     # See the ODBC documentation for more information about this call.
     #
     sub GetForeignKeys {
-        my ($dbh, $PK_Catalog, $PK_Schema, $PK_Table, $FK_Catalog, $FK_Schema, $FK_Table) = @_;
+        my ($dbh, $pk_catalog, $pk_schema, $pk_table,
+            $fk_catalog, $fk_schema, $fk_table) = @_;
         # create a "blank" statement handle
         my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLForeignKeys" });
-        _GetForeignKeys($dbh, $sth, $PK_Catalog, $PK_Schema, $PK_Table, $FK_Catalog, $FK_Schema, $FK_Table) or return;
-        $sth;
+        _GetForeignKeys($dbh, $sth, $pk_catalog, $pk_schema, $pk_table,
+			$fk_catalog, $fk_schema, $fk_table) or return;
+        return $sth;
     }
 
     # Call the ODBC function SQLPrimaryKeys
@@ -402,11 +432,11 @@ $DBD::ODBC::VERSION = '1.21';
     # See the ODBC documentation for more information about this call.
     #
     sub GetPrimaryKeys {
-        my ($dbh, $Catalog, $Schema, $Table) = @_;
+        my ($dbh, $catalog, $schema, $table) = @_;
         # create a "blank" statement handle
         my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLPrimaryKeys" });
-        _GetPrimaryKeys($dbh, $sth, $Catalog, $Schema, $Table) or return;
-        $sth;
+        _GetPrimaryKeys($dbh, $sth, $catalog, $schema, $table) or return;
+        return $sth;
     }
 
     # Call the ODBC function SQLSpecialColumns
@@ -414,11 +444,12 @@ $DBD::ODBC::VERSION = '1.21';
     # See the ODBC documentation for more information about this call.
     #
     sub GetSpecialColumns {
-	my ($dbh, $Identifier, $Catalog, $Schema, $Table, $Scope, $Nullable) = @_;
+	my ($dbh, $identifier, $catalog, $schema, $table, $scope, $nullable) = @_;
 	# create a "blank" statement handle
 	my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLSpecialColumns" });
-	_GetSpecialColumns($dbh, $sth, $Identifier, $Catalog, $Schema, $Table, $Scope, $Nullable) or return;
-	$sth;
+	_GetSpecialColumns($dbh, $sth, $identifier, $catalog, $schema,
+			   $table, $scope, $nullable) or return;
+	return $sth;
     }
 
     sub GetTypeInfo {
@@ -427,7 +458,7 @@ $DBD::ODBC::VERSION = '1.21';
 	my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLGetTypeInfo" });
 	# print "SQL Type is $sqltype\n";
 	_GetTypeInfo($dbh, $sth, $sqltype) or return;
-	$sth;
+	return $sth;
     }
 
     sub type_info_all {
@@ -436,12 +467,11 @@ $DBD::ODBC::VERSION = '1.21';
 	my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLGetTypeInfo" });
 	_GetTypeInfo($dbh, $sth, $sqltype) or return;
 	my $info = $sth->fetchall_arrayref;
-	unshift @$info, {
+	unshift @{$info}, {
 	    map { ($sth->{NAME}->[$_] => $_) } 0..$sth->{NUM_OF_FIELDS}-1
 	};
 	return $info;
     }
-
 }
 
 
@@ -465,16 +495,14 @@ $DBD::ODBC::VERSION = '1.21';
 
     sub ColAttributes { # maps to SQLColAttributes
 	my ($sth, $colno, $desctype) = @_;
-	# print "before ColAttributes $colno\n";
 	my $tmp = _ColAttributes($sth, $colno, $desctype);
-	# print "After ColAttributes\n";
-	$tmp;
+	return $tmp;
     }
 
     sub cancel {
 	my $sth = shift;
 	my $tmp = _Cancel($sth);
-	$tmp;
+	return $tmp;
     }
 }
 
@@ -485,6 +513,10 @@ __END__
 
 DBD::ODBC - ODBC Driver for DBI
 
+=head1 VERSION
+
+This documentation refers to DBD::ODBC version 1.22_2.
+
 =head1 SYNOPSIS
 
   use DBI;
@@ -1216,6 +1248,29 @@ In discussion on the dbi-dev list is was suggested that the ':' could
 be made optional and there were no basic objections but it has not
 made it's way into the pod yet.
 
+=head3 Sticky Parameter Types
+
+The DBI specification post 1.608 says in bind_param:
+
+  The data type is 'sticky' in that bind values passed to execute()
+  are bound with the data type specified by earlier bind_param()
+  calls, if any.  Portable applications should not rely on being able
+  to change the data type after the first C<bind_param> call.
+
+DBD::ODBC does allow a parameter to be rebound with another data type as
+ODBC inherently allows this. Therefore you can do:
+
+  # parameter 1 set as a SQL_LONGVARCHAR
+  $sth->bind_param(1, $data, DBI::SQL_LONGVARCHAR);
+  # without the bind above the $data parameter would be either a DBD::ODBC
+  # internal default or whatever the ODBC driver said it was but because
+  # parameter types are sticky, the type is still SQL_LONGVARCHAR.
+  $sth->execute($data);
+  # change the bound type to SQL_VARCHAR
+  # some DBDs will ignore the type in the following, DBD::ODBC does not
+  $sth->bind_param(1, $data, DBI::SQL_VARCHAR);
+
+
 =head2 Unicode
 
 The ODBC specification supports wide character versions (a postfix of
@@ -1586,7 +1641,20 @@ L<http://www.easysoft.com/developer/languages/perl/tutorial_data_web.html>
 Frequently asked questions are now in L<DBD::ODBC::FAQ>. Run
 C<perldoc DBD::ODBC::FAQ> to view them.
 
-=head1 BUGS
+=head1 CONFIGURATION AND ENVIRONMENT
+
+You should consult the documentation for the ODBC Driver Manager
+you are using.
+
+=head1 DEPENDENCIES
+
+L<DBI>
+
+L<Test::Simple>
+
+=head1 INCOMPATIBILITIES
+
+=head1 BUGS AND LIMITATIONS
 
 None known other than the deviations from the DBI specification mentioned
 above in L</Deviations from the DBI specification>.
@@ -1604,12 +1672,16 @@ Thomas K. Wenrich
 
 Martin J. Evans
 
-=head1 LICENSE
+=head1 LICENSE AND COPYRIGHT
 
 This program is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+it under the same terms as Perl itself. See L<perlartistic>. This
+program is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE.
 
-See L<http://www.perl.com/perl/misc/Artistic.html>
+Portions of this software are Copyright Tim Bunce, Thomas K. Wenrich,
+Jeff Urlwin and Martin J. Evans - see the source.
 
 =head1 SEE ALSO
 
